// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of paramxx
//
// paramxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// paramxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <stdlib.h>
#include <suplib/str.h>

#include "StringPar.h"

StringPar::StringPar( ParTxt& par ) throw ( ParFileException ) : Par( par ) {

  try {

    //
    // The classes BoolPar, LongPar, RealPar must be validated
    // but a string can be anything so there is no need to validate.
    //

    between_limits( parameter[ PARVALUE ] );

  } catch ( ParFileException& pfe ) {
    throw;
  }

}

void StringPar::between_limits( const string& str ) const
  throw ( ParFileException, Exception ) {

    // if the value field has an indirrection then do not check for limits
    if ( Par::is_indirrect( str ) )
      return;

    if ( "" != parameter[ PARMINIMUM ] ) {

      if ( string::npos != parameter[ PARMINIMUM ].find( str ) ) {
	char* ptr = (char*) parameter[ PARMINIMUM ].c_str( );
	int restore = 1;
	char** buffer_argv = NULL;
	size_t num_tokens = str_tokenize( ptr, "|", &buffer_argv, 0, restore );
	for ( int ii = 0; ii < num_tokens; ii++ ) {
	  if ( 0 == str.compare( buffer_argv[ ii ] ) ) {
	    str_tokenize_free( buffer_argv, restore );
	    return;
	  }
	}
	str_tokenize_free( buffer_argv, restore );
      } 

      {
	char tmp[ 256 ];
	
	const char* format =
	  "StringPar::between_limits( %s ) : For parameter `%s', "
	  "the value `%s' must be one of `";

	sprintf( tmp, format, str.c_str( ), parameter[ PARNAME ].c_str( ),
		 str.c_str( ) ); 

	// since parameter[ PARMINIMUM ] may contain %
	string msg( tmp );

	msg += parameter[ PARMINIMUM ];
	msg += "'\n";

	throw ParFileException( msg );

      }

    }

}

void StringPar::print( ostream& os ) const {

  for ( int i = 0; i < Par::NUMTOKENS; i++ ) {
    if ( i == Par::PARMODE )
      os << '"';
    os << parameter[ i ];
    if ( i == Par::PARMODE )
      os << '"';
    if ( i != Par::NUMTOKENS - 1 )
      os << char( DELIMIT );
  }

}

void StringPar::set_val( const string& str ) throw ( ParFileException ,
						     Exception ) {

  try {

    between_limits( str );
    parameter[ PARVALUE ] = str;

  } catch ( ParFileException& pfe ) {

    throw;

  } catch ( Exception& e ) {

    throw;

  } 

}

