#include <cstdlib>
#include <paramxx/ParFile.h>

using namespace std;

/*
  PROCEDURE
    parlist - list parameters from one or more IRAF-style  parameter files

  SYNOPSIS
    parlist [ paramfile1 ... ]

  DESCRIPTION
    plist  displays  the  parameters  from  each  paramfile   in
    sequence on the standard output.  Thus:

         example% plist src

    displays the contents of the src parameter file on the stan-
    dard output:

    A .par extension will be added to the paramfile name, if  no
    extension  is  specified.  If the file is not located in the
    directory specified by the paramfile, the directory  pointed
    to by the UPARM environment variable will be searched.

  OPTIONS
     none

  ENVIRONMENT
    The environment variable PFILES specifies the list of directories,
    delimited by colon( :), in which to look for the parameter file.

    The environment variable UPARM specifies  the  directory  in
    which to look for the parameter file, if the pathname of the
    file is not part of the paramfile specification.
*/

int parlist( Par* par ) {
  par->plist( cout );
  cout << endl;
  return 0;
}

static int parlist( int argc, char** argv ) {

  try {

    for ( int i = 1; i < argc; i++ ) {
      ParFile pfile( argc, argv, argv[ i ] );
      string name = pfile.get_filename();
      cout << "\nParameters for " << name << "\n\n";
      pfile.traverse( parlist );
    }

    return EXIT_SUCCESS;

  } catch( ParFileException& pfe ) {
    cerr << pfe << endl;
    return EXIT_FAILURE;
  } catch( Exception& e ) {
    cerr << e << endl;
    return EXIT_FAILURE;
  }

}

int main( int argc, char** argv ) {

  if ( argc < 2 ) {
    cerr << "Usage:\t" << argv[0] << " pfile(s)\n";
    return argc;
  }

  return parlist( argc, argv );
}

