#include <stdlib.h>
#include <testlib/testlib.h>

#include <paramxx/BoolPar.h>

#define MYLENGTH 128
static char mystr[ MYLENGTH ];

static int testBoolPar1( bool printIt ) {

  static char* yep[] = { "yes", "y", "1", NULL };
  static char* nope[] = { "no", "n", "0", NULL };

  try {

    strcpy( mystr, "reuse_img,b,a,no,,,\"reuse intermediate image?\"" );
    ParTxt parTxt( mystr );
    BoolPar boolPar( parTxt );

    {
      int ii = 0;
      while( yep[ ii ] ) {
	boolPar.set_val( yep[ ii ] );
	bool result = boolPar.pgetb( );
	TEST( "BoolPar set_val and pgetb", result, true );
	ii++;
      }
    }

    {
      int ii = 0;
      while( nope[ ii ] ) {
	boolPar.set_val( nope[ ii ] );
	bool result = boolPar.pgetb( );
	TEST( "BoolPar set_val and pgetb", result, false );
	ii++;
      }
    }

    return EXIT_SUCCESS;

  } catch( ParFileException& pfe ) {

    if ( printIt )
      cerr << pfe;
    return EXIT_FAILURE;

  } catch ( Exception & e ) {

    if ( printIt )
      cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  return EXIT_FAILURE;

}

static int illegal_min( bool printIt ) {

  try {

    strcpy( mystr, "reuse_img,b,a,no,a,,\"reuse intermediate image?\"" );
    ParTxt parTxt( mystr );
    BoolPar boolPar( parTxt );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "BoolPar illegal_min( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "BoolPar illegal_min, illegal min", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int illegal_max( bool printIt ) {

  try {

    strcpy( mystr, "reuse_img,b,a,no,,aaa,\"reuse intermediate image?\"" );
    ParTxt parTxt( mystr );
    BoolPar boolPar( parTxt );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "BoolPar illegal_max( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "BoolPar illegal_max, illegal max", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_int_type( bool printIt ) {

  try {

    strcpy( mystr, "reuse_img,b,a,no,,,\"reuse intermediate image?\"" );
    ParTxt parTxt( mystr );
    BoolPar boolPar( parTxt );

    int foo = boolPar.pgeti( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "BoolPar not_int_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "BoolPar not_int_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_long_type( bool printIt ) {

  try {

    strcpy( mystr, "reuse_img,b,a,no,,,\"reuse intermediate image?\"" );
    ParTxt parTxt( mystr );
    BoolPar boolPar( parTxt );

    long foo = boolPar.pgetl( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "BoolPar not_long_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "BoolPar not_long_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_double_type( bool printIt ) {

  try {

    strcpy( mystr, "reuse_img,b,a,no,,,\"reuse intermediate image?\"" );
    ParTxt parTxt( mystr );
    BoolPar boolPar( parTxt );

    double foo = boolPar.pgetd( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "BoolPar not_double_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "BoolPar not_double_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_str_type( bool printIt ) {

  try {

    strcpy( mystr, "reuse_img,b,a,no,,,\"reuse intermediate image?\"" );
    ParTxt parTxt( mystr );
    BoolPar boolPar( parTxt );

    boolPar.pgetstr( mystr, MYLENGTH );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "BoolPar not_str_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "BoolPar not_str_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_string_type( bool printIt ) {

  try {

    strcpy( mystr, "reuse_img,b,a,no,,,\"reuse intermediate image?\"" );
    ParTxt parTxt( mystr );
    BoolPar boolPar( parTxt );

    string foo = boolPar.pgetstring( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "BoolPar not_string_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "BoolPar not_string_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}


// 0 ==> true
// 1 ==> false
static int minIsBiggerThenMax( bool printIt ) {
  // not yet implemented in the library.
  return 0;
}

int testBoolPar( bool printIt ) {

  if ( EXIT_FAILURE == testBoolPar1( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == illegal_min( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == illegal_max( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_int_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_long_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_double_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_str_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_string_type( printIt ) )
    return EXIT_FAILURE;

  /*
  if ( EXIT_FAILURE == minIsBiggerThenMax( ) )
    return EXIT_FAILURE;
    */

  return EXIT_SUCCESS;

}
