#include <stdlib.h>
#include <testlib/testlib.h>

#include <paramxx/CommentPar.h>

#define MYLENGTH 128
static char mystr[ MYLENGTH ];

static int tstCommentPar( bool printIt ) {

  try {

    strcpy( mystr, "# now is the time for all good men" );
    ParTxt pound( mystr );
    CommentPar poundSign( pound );

    strcpy( mystr, "\n" );
    ParTxt blank( mystr );

    CommentPar blankLine( blank );

    if ( printIt ) {
      cout << poundSign << '\n';
      cout << blankLine << '\n';
    }

    // One should not be able to set a value to a CommentPar.
    // Must catch the ParFileException thrown by the library.
    poundSign.set_val( "foo" );

  } catch( ParFileException& pfe ) {

    TEST( "CommentPar cannot use set_val", EXIT_SUCCESS, EXIT_SUCCESS );
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  TEST( "Cannot use set_val to a commentPar", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_bool_type( bool printIt ) {

  try {

    strcpy( mystr, "# now is the time for all good men" );
    ParTxt pound( mystr );
    CommentPar commentPar( pound );

    bool foo = commentPar.pgetb( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "CommentPar not_bool_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "CommentPar not_bool_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_int_type( bool printIt ) {

  try {

    strcpy( mystr, "# now is the time for all good men" );
    ParTxt pound( mystr );
    CommentPar commentPar( pound );

    int foo = commentPar.pgeti( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "CommentPar not_int_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "CommentPar not_int_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_long_type( bool printIt ) {

  try {

    strcpy( mystr, "# now is the time for all good men" );
    ParTxt pound( mystr );
    CommentPar commentPar( pound );

    long foo = commentPar.pgetl( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "CommentPar not_long_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "CommentPar not_long_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_double_type( bool printIt ) {

  try {

    strcpy( mystr, "# now is the time for all good men" );
    ParTxt pound( mystr );
    CommentPar commentPar( pound );

    double foo = commentPar.pgetd( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "CommentPar not_double_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "CommentPar not_double_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_str_type( bool printIt ) {

  try {

    strcpy( mystr, "# now is the time for all good men" );
    ParTxt pound( mystr );
    CommentPar commentPar( pound );

    commentPar.pgetstr( mystr, MYLENGTH );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "CommentPar not_str_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "CommentPar not_str_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_string_type( bool printIt ) {

  try {

    strcpy( mystr, "# now is the time for all good men" );
    ParTxt pound( mystr );
    CommentPar commentPar( pound );

    string foo = commentPar.pgetstring( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "CommentPar not_string_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "CommentPar not_string_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

int testCommentPar( bool printIt ) {

  if ( EXIT_FAILURE == tstCommentPar( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_bool_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_int_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_long_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_double_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_str_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_string_type( printIt ) )
    return EXIT_FAILURE;

  return EXIT_SUCCESS;

}
